@@ -55,9 +55,15 @@ __mrss_free_channel (mrss_t * mrss)
   if (mrss->title)
     free (mrss->title);
 
+  if (mrss->title_type)
+    free (mrss->title_type);
+
   if (mrss->description)
     free (mrss->description);
 
+  if (mrss->description_type)
+    free (mrss->description_type);
+
   if (mrss->link)
     free (mrss->link);
 
@@ -73,6 +79,9 @@ __mrss_free_channel (mrss_t * mrss)
   if (mrss->copyright)
     free (mrss->copyright);
 
+  if (mrss->copyright_type)
+    free (mrss->copyright_type);
+
   if (mrss->pubDate)
     free (mrss->pubDate);
 
@@ -329,15 +338,24 @@ __mrss_free_item (mrss_item_t * item)
   if (item->title)
     free (item->title);
 
+  if (item->title_type)
+    free (item->title_type);
+
   if (item->link)
     free (item->link);
 
   if (item->description)
     free (item->description);
 
+  if (item->description_type)
+    free (item->description_type);
+
   if (item->copyright)
     free (item->copyright);
 
+  if (item->copyright_type)
+    free (item->copyright_type);
+
   if (item->author)
     free (item->author);
 
