@@ -19,8 +19,6 @@
 #ifndef __M_RSS_INTERNAL_H__
 #define __M_RSS_INTERNAL_H__
 
-#define _GNU_SOURCE
-
 #include <curl/curl.h>
 #include <nxml.h>
 #include <sys/types.h>
@@ -28,9 +26,17 @@
 #include <string.h>
 #include <errno.h>
 
-#include <locale.h>
+#ifdef USE_LOCALE
+#  ifdef USE_X_LOCALE
+#    include <xlocale.h>
+#  endif
 
-char *	__mrss_download_file	(nxml_t *, char *, size_t *);
+#  ifdef USE_GENERIC_LOCALE
+#    include <locale.h>
+#  endif
+#endif
+
+char *	__mrss_download_file	(nxml_t *, char *, size_t *, mrss_error_t *, CURLcode *code);
 
 #endif
 
