@@ -132,6 +132,122 @@ __mrss_parse_tag (nxml_t * doc, nxml_data_t * cur)
   return tag;
 }
 
+static void
+__mrss_parser_atom_string (nxml_t * doc, nxml_data_t * cur, char **what,
+			   char **type)
+{
+  char *c;
+
+  if (!(c = nxmle_find_attribute (cur, "type", NULL)) || !strcmp (c, "text"))
+    {
+      *what = nxmle_get_string (cur, NULL);
+      *type = c;
+      return;
+    }
+
+  if (!strcmp (c, "html") || !strcmp (c, "xhtml"))
+    {
+      nxml_data_t *ncur;
+      char *total, *c1;
+      nxml_t *new;
+      int size;
+
+      total = NULL;
+      size = 0;
+
+      c1 = nxmle_get_string (cur, NULL);
+
+      if (c1 && *c1)
+	{
+	  total = strdup (c1);
+	  size = strlen (total);
+	}
+
+      else
+	{
+	  while ((ncur = cur->children))
+	    {
+	      char *buffer = NULL, *p;
+	      char *tmp;
+	      int len;
+
+	      if (nxml_remove (doc, cur, ncur) != NXML_OK)
+		continue;
+
+	      if (nxml_new (&new) != NXML_OK)
+		{
+		  nxml_free_data (ncur);
+		  continue;
+		}
+
+	      if (nxml_add (new, NULL, &ncur) != NXML_OK)
+		{
+		  nxml_free_data (ncur);
+		  nxml_free (new);
+		  continue;
+		}
+
+	      if (!(buffer = nxmle_write_buffer (new, NULL)))
+		{
+		  nxml_free (new);
+		  continue;
+		}
+
+	      nxml_free (new);
+
+	      if (strncmp (buffer, "<?xml ", 6))
+		{
+		  free (buffer);
+		  continue;
+		}
+
+	      p = buffer;
+
+	      while (*p && *p != '>')
+		p++;
+
+	      if (!*p)
+		{
+		  free (buffer);
+		  continue;
+		}
+
+	      p++;
+	      while (*p && (*p == ' ' || *p == '\t' || *p == '\n'))
+		p++;
+
+	      len = strlen (p);
+
+	      if (!(tmp = realloc (total, size + len + 1)))
+		{
+		  free (buffer);
+
+		  if (total)
+		    {
+		      free (total);
+		      total = NULL;
+		    }
+
+		  break;
+		}
+
+	      total = tmp;
+	      strcpy (total + size, p);
+	      size += len;
+
+	      free (buffer);
+	    }
+	}
+
+      *what = total;
+      *type = c;
+      return;
+    }
+
+  free (c);
+  *what = nxmle_get_string (cur, NULL);
+}
+
 static char *
 __mrss_atom_prepare_date (mrss_t * data, char *datestr)
 {
@@ -232,7 +348,6 @@ __mrss_parser_atom_entry (nxml_t * doc, nxml_data_t * cur, mrss_t * data)
   char *c;
   mrss_item_t *item;
 
-
   if (!(item = malloc (sizeof (mrss_item_t))))
     return;
 
@@ -245,9 +360,9 @@ __mrss_parser_atom_entry (nxml_t * doc, nxml_data_t * cur, mrss_t * data)
       if (cur->type == NXML_TYPE_ELEMENT)
 	{
 	  /* title -> title */
-	  if (!item->title && !strcmp (cur->value, "title")
-	      && (c = nxmle_get_string (cur, NULL)))
-	    item->title = c;
+	  if (!item->title && !strcmp (cur->value, "title"))
+	    __mrss_parser_atom_string (doc, cur, &item->title,
+				       &item->title_type);
 
 	  /* link href -> link */
 	  else if (!item->link && !strcmp (cur->value, "link")
@@ -255,19 +370,19 @@ __mrss_parser_atom_entry (nxml_t * doc, nxml_data_t * cur, mrss_t * data)
 	    item->link = c;
 
 	  /* content -> description */
-	  else if (!item->description && !strcmp (cur->value, "content")
-		   && (c = nxmle_get_string (cur, NULL)))
-	    item->description = c;
+	  else if (!item->description && !strcmp (cur->value, "content"))
+	    __mrss_parser_atom_string (doc, cur, &item->description,
+				       &item->description_type);
 
 	  /* summary -> description */
-	  else if (!item->description && !strcmp (cur->value, "summary")
-		   && (c = nxmle_get_string (cur, NULL)))
-	    item->description = c;
+	  else if (!item->description && !strcmp (cur->value, "summary"))
+	    __mrss_parser_atom_string (doc, cur, &item->description,
+				       &item->description_type);
 
 	  /* right -> copyright */
-	  else if (!item->copyright && !strcmp (cur->value, "rights")
-		   && (c = nxmle_get_string (cur, NULL)))
-	    item->copyright = c;
+	  else if (!item->copyright && !strcmp (cur->value, "rights"))
+	    __mrss_parser_atom_string (doc, cur, &item->description,
+				       &item->description_type);
 
 	  /* author structure -> author elements */
 	  else if (!strcmp (cur->value, "author"))
@@ -336,7 +451,6 @@ __mrss_parser_atom_entry (nxml_t * doc, nxml_data_t * cur, mrss_t * data)
 
       tmp->next = item;
     }
-
 }
 
 static void
@@ -691,23 +805,22 @@ __mrss_parser_atom (nxml_t * doc, nxml_data_t * cur, mrss_t ** ret)
       if (cur->type == NXML_TYPE_ELEMENT)
 	{
 	  /* title -> title */
-	  if (!strcmp (cur->value, "title") && !data->title
-	      && (c = nxmle_get_string (cur, NULL)))
-	    data->title = c;
+	  if (!data->title && !strcmp (cur->value, "title"))
+	    __mrss_parser_atom_string (doc, cur, &data->title,
+				       &data->title_type);
 
 	  /* subtitle -> description */
-	  else if (!strcmp (cur->value, "subtitle")
+	  else if (!data->description
 		   && data->version == MRSS_VERSION_ATOM_1_0
-		   && !data->description
-		   && (c = nxmle_get_string (cur, NULL)))
-	    data->description = c;
+		   && !strcmp (cur->value, "subtitle"))
+	    __mrss_parser_atom_string (doc, cur, &data->description,
+				       &data->description_type);
 
 	  /* tagline -> description (Atom 0.3) */
-	  else if (!strcmp (cur->value, "tagline")
-		   && data->version == MRSS_VERSION_ATOM_0_3
-		   && !data->description
-		   && (c = nxmle_get_string (cur, NULL)))
-	    data->description = c;
+	  else if (data->version == MRSS_VERSION_ATOM_0_3
+		   && !data->description && !strcmp (cur->value, "tagline"))
+	    __mrss_parser_atom_string (doc, cur, &data->description,
+				       &data->description_type);
 
 	  /* link href -> link */
 	  else if (!strcmp (cur->value, "link") && !data->link
@@ -720,9 +833,9 @@ __mrss_parser_atom (nxml_t * doc, nxml_data_t * cur, mrss_t ** ret)
 	    data->id = c;
 
 	  /* rights -> copyright */
-	  else if (!strcmp (cur->value, "rights") && !data->copyright
-		   && (c = nxmle_get_string (cur, NULL)))
-	    data->copyright = c;
+	  else if (!data->copyright && !strcmp (cur->value, "rights"))
+	    __mrss_parser_atom_string (doc, cur, &data->copyright,
+				       &data->copyright_type);
 
 	  /* updated -> lastBuildDate */
 	  else if (!strcmp (cur->value, "updated")
@@ -1078,26 +1191,50 @@ __mrss_parser (nxml_t * doc, mrss_t ** ret)
 mrss_error_t
 mrss_parse_url (char *url, mrss_t ** ret)
 {
-  return mrss_parse_url_with_options_and_error (url, ret, NULL, NULL);
+  return mrss_parse_url_with_options_error_and_transfer_buffer (url, ret,
+								NULL, NULL,
+								NULL, NULL);
 }
 
 mrss_error_t
 mrss_parse_url_with_options (char *url, mrss_t ** ret,
 			     mrss_options_t * options)
 {
-  return mrss_parse_url_with_options_and_error (url, ret, options, NULL);
+  return mrss_parse_url_with_options_error_and_transfer_buffer (url, ret,
+								options, NULL,
+								NULL, NULL);
 }
 
 mrss_error_t
 mrss_parse_url_with_options_and_error (char *url, mrss_t ** ret,
 				       mrss_options_t * options,
 				       CURLcode * code)
+{
+  return mrss_parse_url_with_options_error_and_transfer_buffer (url, ret,
+								options, code,
+								NULL, NULL);
+}
+
+mrss_error_t
+mrss_parse_url_with_options_error_and_transfer_buffer (char *url,
+						       mrss_t ** ret,
+						       mrss_options_t *
+						       options,
+						       CURLcode * code,
+						       char **feed_content,
+						       int *feed_size)
 {
   nxml_t *doc;
   mrss_error_t err;
   char *buffer;
   size_t size;
 
+  if (feed_content)
+    *feed_content = NULL;
+
+  if (feed_size)
+    *feed_size = 0;
+
   if (!url || !ret)
     return MRSS_ERR_DATA;
 
@@ -1137,10 +1274,9 @@ mrss_parse_url_with_options_and_error (char *url, mrss_t ** ret,
     {
       if (!((*ret)->file = strdup (url)))
 	{
-	  free (buffer);
-
 	  mrss_free (*ret);
 	  nxml_free (doc);
+	  free (buffer);
 
 	  return MRSS_ERR_POSIX;
 	}
@@ -1148,9 +1284,17 @@ mrss_parse_url_with_options_and_error (char *url, mrss_t ** ret,
       (*ret)->size = size;
     }
 
-  free (buffer);
   nxml_free (doc);
 
+  /* transfer ownership: */
+  if (!feed_content)
+    free (buffer);
+  else
+    *feed_content = buffer;
+
+  if (feed_size)
+    *feed_size = size;
+
   return err;
 }
 
