@@ -100,7 +100,7 @@ __mrss_write_string (void (*func) (void *, char *, ...), void *obj, char *str)
 	  __NXML_CHECK_BUF;
 	}
 
-      else if (str[i] == '\'')
+      else if (str[i] == '\"')
 	{
 	  buf[j++] = '&';
 	  __NXML_CHECK_BUF;
@@ -664,7 +664,16 @@ __mrss_write_real_atom_entry (mrss_t * mrss,
 
       if (item->title)
 	{
-	  func (obj, "    <title>");
+	  func (obj, "    <title");
+
+	  if (item->title_type)
+	    {
+	      func (obj, " type=\"");
+	      __mrss_write_string (func, obj, item->title_type);
+	      func (obj, "\"");
+	    }
+
+	  func (obj, ">");
 	  __mrss_write_string (func, obj, item->title);
 	  func (obj, "</title>\n");
 	}
@@ -678,7 +687,16 @@ __mrss_write_real_atom_entry (mrss_t * mrss,
 
       if (item->description)
 	{
-	  func (obj, "    <summary>");
+	  func (obj, "    <summary");
+
+	  if (item->description_type)
+	    {
+	      func (obj, " type=\"");
+	      __mrss_write_string (func, obj, item->description_type);
+	      func (obj, "\"");
+	    }
+
+	  func (obj, ">");
 	  __mrss_write_string (func, obj, item->description);
 	  func (obj, "</summary>\n");
 	}
@@ -787,7 +805,16 @@ __mrss_write_atom (mrss_t * mrss, void (*func) (void *, char *, ...),
 
   func (obj, ">\n");
 
-  func (obj, "  <title>");
+  func (obj, "  <title");
+
+  if (mrss->title_type)
+    {
+      func (obj, " type=\"");
+      __mrss_write_string (func, obj, mrss->title_type);
+      func (obj, "\"");
+    }
+
+  func (obj, ">");
   __mrss_write_string (func, obj, mrss->title);
   func (obj, "</title>\n");
 
@@ -795,13 +822,31 @@ __mrss_write_atom (mrss_t * mrss, void (*func) (void *, char *, ...),
     {
       if (mrss->version == MRSS_VERSION_ATOM_1_0)
 	{
-	  func (obj, "  <subtitle>");
+	  func (obj, "  <subtitle");
+
+	  if (mrss->description_type)
+	    {
+	      func (obj, " type=\"");
+	      __mrss_write_string (func, obj, mrss->description_type);
+	      func (obj, "\"");
+	    }
+
+	  func (obj, ">");
 	  __mrss_write_string (func, obj, mrss->description);
 	  func (obj, "</subtitle>\n");
 	}
       else
 	{
-	  func (obj, "  <tagline>");
+	  func (obj, "  <tagline");
+
+	  if (mrss->description_type)
+	    {
+	      func (obj, " type=\"");
+	      __mrss_write_string (func, obj, mrss->description_type);
+	      func (obj, "\"");
+	    }
+
+	  func (obj, ">");
 	  __mrss_write_string (func, obj, mrss->description);
 	  func (obj, "</tagline>\n");
 	}
@@ -823,7 +868,16 @@ __mrss_write_atom (mrss_t * mrss, void (*func) (void *, char *, ...),
 
   if (mrss->copyright)
     {
-      func (obj, "  <rights>");
+      func (obj, "  <rights");
+
+      if (mrss->copyright_type)
+	{
+	  func (obj, " type=\"");
+	  __mrss_write_string (func, obj, mrss->copyright_type);
+	  func (obj, "\"");
+	}
+
+      func (obj, ">");
       __mrss_write_string (func, obj, mrss->copyright);
       func (obj, "</rights>\n");
     }
